//********************************************************************
UNIT ButtonBarUnit;
{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Jonas Scholz (i0n0s)
  jonasscholz@web.de
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2009 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications,
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original
  software. If you use this software in a product, an acknowledgment in the product documentation would be
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  ButtonBar for the easySDL-framework

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  Created January 2009 (GT)

  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors:

  GT
    Gertraude Trawnicek (Traude)
    send me a PM at www.delphigl.com
    
*******************************************************************************)

INTERFACE

USES Classes, Contnrs, easySDLFont, dglOpenGl;

TYPE
   TRGBAColor = Packed Record Red,Green,Blue,Alpha: Single; End;
   TBtnBarEvent = Procedure(ASender: TObject; AIndex: Integer) Of Object;

   TPushButton = Class(TObject)
   Private
      fCaption: String;
      fLeft,fTop,fRight,fBottom: Integer;
   Public
      Function Hit(MouseX,MouseY: Integer): Boolean;
   End;

   TButtonBar = Class(TObject)
   Private
      fOnClick: TBtnBarEvent;

      fCurrentButton: Integer;
      fColor: TRGBAColor;
      fPushColor: TRGBAColor;
      fButton: TObjectList;
      fBarLeft,fBarTop,fBarRight,fBarBottom: Integer;
      fFont: TFont; // EasySDLFont
      fSpace: Byte;
      fMargin: Byte;

      Function Hit(MouseX,MouseY: Integer): Integer;
   Public
      Property OnClick: TBtnBarEvent Write fOnClick;

      Constructor Create(AColor: TRGBAColor;
               ALeft,ATop: Integer; AFont: TFont); Reintroduce;
      Destructor Destroy; Override;
      Procedure AddButton(ACaption: String);
      Procedure MouseBtnLDown(MouseX,MouseY: Integer);
      Procedure Render;
      Procedure SetIndex(AIndex: Integer);
   End;
//********************************************************************
IMPLEMENTATION
//********************************************************************
Function TPushButton.Hit(MouseX,MouseY: Integer): Boolean;
Begin
   Result:= FALSE;

   If (MouseX >= fLeft)    And (MouseX <= fRight) And
      (MouseY >= fBottom)  And (MouseY <= fTop)
      Then Result:= TRUE;
End;
//********************************************************************
//********************************************************************
Constructor TButtonBar.Create(AColor: TRGBAColor;
               ALeft,ATop: Integer; AFont: TFont);
Begin
   Inherited Create;

   fColor:= AColor;

   With fPushColor Do Begin
        Red:= 255;
      Green:= 0;
       Blue:= 0;
      Alpha:= 255;
   End;

   fButton:= TObjectList.Create;

   fFont:= AFont;

   fSpace:= 5;
   fMargin:= 5;

     fBarLeft:= ALeft;
      fBarTop:= ATop;
    fBarRight:= fBarLeft;
   fBarBottom:= fBarTop - fFont.Height - 2*fMargin;

   fCurrentButton:= -1;
End;
//********************************************************************
Destructor TButtonBar.Destroy;
Begin
   fFont.Free;
   fButton.Free;
   Inherited Destroy;
End;
//********************************************************************
Procedure TButtonBar.AddButton(ACaption: String);
Var LocalButton: TPushButton;
Begin
   If Length(ACaption) > 0 Then Begin

      LocalButton:= TPushButton.Create;

      With LocalButton Do Begin
         fCaption:= ACaption;
         fTop:= fBarTop;
         fBottom:= fBarBottom;

         fLeft:= fBarRight;
         fRight:= fLeft + fFont.GetWidth(fCaption) + 2*fMargin;

         fBarRight:= fRight + fSpace;
      End;

      fButton.Add(LocalButton);

      If fCurrentButton < 0
         Then fCurrentButton:= 0;
   End;
End;
//********************************************************************
Function TButtonBar.Hit(MouseX,MouseY: Integer): Integer;
Var
   LocalButton: TPushButton;
   Index: Integer;
   ButtonHit: Boolean;
Begin
   Result:= fCurrentButton;

   If (MouseX >= fBarLeft) And (MouseX <= fBarRight) And
      (MouseY >= fBarBottom)  And (MouseY <= fBarTop)

      Then If fButton.Count > 0 Then Begin
         ButtonHit:= FALSE;
         Index:= 0;
         While (Index < fButton.Count) And (Not ButtonHit) Do Begin
            LocalButton:= TPushButton(fButton[Index]);
            ButtonHit:= LocalButton.Hit(MouseX, MouseY);
            Inc(Index);
         End;
         Dec(Index);

         If ButtonHit Then Begin
            fCurrentButton:= Index;
            Result:= fCurrentButton;
         End;
      End;
End;
//********************************************************************
Procedure TButtonBar.MouseBtnLDown(MouseX,MouseY: Integer);
Var Index: Integer;
Begin
   Index:= Hit(MouseX,MouseY);
   If (Index >= 0) And (Assigned(fOnClick))
      Then fOnClick(Self,Index);
End;
//********************************************************************
Procedure TButtonBar.Render;
Var
   LocalButton: TPushButton;
   Index: Integer;
Begin
   If fButton.Count > 0 Then Begin

      //**************************************************************
      glBegin(GL_QUADS);
         For Index:= 0 To fButton.Count-1 Do Begin

            If Index = fCurrentButton
               Then With fPushColor Do glColor4f(Red,Green,Blue,Alpha)
               Else With fColor Do glColor4f(Red,Green,Blue,Alpha);

            LocalButton:= TPushButton(fButton[Index]);
            With LocalButton Do Begin
               glVertex2f(fLeft, fBottom);
               glVertex2f(fRight,fBottom);
               glVertex2f(fRight,fTop);
               glVertex2f(fLeft, fTop);
            End;
         End;
      glEnd;
      //**************************************************************

      For Index:= 0 To fButton.Count-1 Do Begin
         LocalButton:= TPushButton(fButton[Index]);
         With LocalButton Do
            fFont.Draw(fCaption,fLeft+fMargin,fBottom+fMargin);
      End;
   End;
End;
//********************************************************************
Procedure TButtonBar.SetIndex(AIndex: Integer);
Begin
   With fButton Do If Count > 0 Then Begin
      If (AIndex >= 0) And (AIndex < Count)
         Then fCurrentButton:= AIndex;
   End;
End;
//********************************************************************
End.

