program template;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ELSE}
  {$APPTYPE CONSOLE}
{$ENDIF}

{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Jonas Scholz (i0n0s)
  jonasscholz@web.de
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2009 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  Source code for Tutorial 8 (OpenGL Lighting): GT

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------

  Specific lighting source code (created January 2009): GT
  Template: JS

  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors:


  JS
    Jonas Scholz (i0n0s)
    email : jonasscholz@web.de
  GT
    Gertraude Trawnicek (Traude)
    send me a PM at www.delphigl.com

*******************************************************************************)

uses
  SysUtils,
  dglOpenGL,
  SDL,
  easySDL,
  easySDLFont,
  easySDLTextures,
  ButtonBarUnit,
  Shape3DUnit;

{$I ../dglsdk.inc}

Const

   LIGHT_NONE = 0;
   LIGHT_ORDINARY = 1;
   LIGHT_PER_SPOT = 2;
   LIGHT_PER_SPHEREMAP = 3;

//----------------------------------------------------------------------------
// Licht-Einstellungen
//----------------------------------------------------------------------------

   // Light Model
   Ambient_Model  : Array[0..3] Of Single = (0.2, 0.2, 0.2, 1.0);
   Reflect_Model  : Single = 0;

   // Material
   Mat_Specular   : Array[0..3] Of Single = (0.8, 0.8, 0.8, 1.0);
   Mat_Shininess  : Array[0..0] Of Single = (50.0);
   Mat_Ambient    : Array[0..3] Of Single = (0.8, 0.8, 0.8, 1.0);
   Mat_Diffuse    : Array[0..3] Of Single = (0.8, 0.8, 0.8, 1.0);

   // Normal Light
   Light_Position : Array[0..3] Of Single = (-4, 0, 0, 1);
   Light_Ambient  : Array[0..3] Of Single = (0.8, 0.8, 0.8, 1.0);
   Light_Diffuse  : Array[0..3] Of Single = (0.8, 0.8, 0.8, 1.0);

   // Spotlight
   SpotL_Position : Array[0..3] Of Single = (  0,   0,   0, 1);
   SpotL_Direction: Array[0..2] Of Single = (0.4, 0.0,-1.0);


type
  TMyProgram = class(TEasySDL)
    FPSFont: TFont;
    Procedure DrawScene; override;
    procedure HandleMousePressed(event: TSDL_MouseButtonEvent); Override;
    Procedure ChangeSwitch(ASender: TObject; AIndex: Integer);
   End;

var
    MyProgram   : TMyProgram;
    MyButtonBar : TButtonBar;
    ButtonColor : TRGBAColor;
    MyShape3D: TShape3D;
    Switch: Byte;
    Angle: Word;

//----------------------------------------------------------------------------
// OpenGL-Einstellungen
//----------------------------------------------------------------------------
Procedure SetOpenGLParams;
Begin
   glEnable(GL_DEPTH_TEST);
   glDepthFunc(GL_LEQUAL);
   glEnable(GL_Texture_2D);
   glEnable(GL_CULL_FACE);
   glCullFace(GL_BACK);
   glPolygonMode(GL_FRONT,GL_FILL);    // Filled mode
   //glPolygonMode(GL_FRONT,GL_LINE);  // Wireframe mode
   glClearColor(0,0,0,1);

   // Light Model
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, @Ambient_Model[0]);
   glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, @Reflect_Model);

   // Material
   glMaterialfv(GL_FRONT, GL_SPECULAR,  @Mat_Specular[0]);
   glMaterialfv(GL_FRONT, GL_SHININESS, @Mat_Shininess[0]);
   glMaterialfv(GL_FRONT, GL_AMBIENT,   @Mat_Ambient[0]);
   glMaterialfv(GL_FRONT, GL_DIFFUSE,   @Mat_Diffuse[0]);

End;
//----------------------------------------------------------------------------
// Licht-Einstellungen
//----------------------------------------------------------------------------
Procedure TurnLightOFF;
Begin
   glDisable(GL_COLOR_MATERIAL);
   glDisable(GL_LIGHTING);
   glDisable(GL_LIGHT0);
   glDisable(GL_LIGHT1);

   glDisable(GL_TEXTURE_GEN_S);
   glDisable(GL_TEXTURE_GEN_T);
End;
//----------------------------------------------------------------------------
Procedure TurnLightON;
Begin
   Case Switch Of

      //-----------------------------------------------------------
      LIGHT_NONE:; // Nothing To Do

      //-----------------------------------------------------------
      LIGHT_ORDINARY: Begin
         glLightfv(GL_LIGHT0, GL_AMBIENT,  @Light_Ambient[0]);
         glLightfv(GL_LIGHT0, GL_DIFFUSE,  @Light_Diffuse[0]);
         glLightfv(GL_LIGHT0, GL_POSITION, @Light_Position[0]);

         glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.04);
         glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0);
         glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.01);

         glEnable(GL_COLOR_MATERIAL);
         glEnable(GL_LIGHTING);
         glEnable(GL_LIGHT0);
      End;

      //-----------------------------------------------------------
      LIGHT_PER_SPOT: Begin
         glLightfv(GL_LIGHT1, GL_AMBIENT,  @Light_Ambient[0]);
         glLightfv(GL_LIGHT1, GL_DIFFUSE,  @Light_Diffuse[0]);
         glLightfv(GL_LIGHT1, GL_POSITION, @SpotL_Position[0]);

         glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, @SpotL_Direction[0]);
         glLightf(GL_LIGHT1, GL_SPOT_CUTOFF, 20.0);
         glLightf(GL_LIGHT1, GL_SPOT_EXPONENT, 10.0);

         glEnable(GL_COLOR_MATERIAL);
         glEnable(GL_LIGHTING);
         glEnable(GL_LIGHT1);
      End;

      //-----------------------------------------------------------
      LIGHT_PER_SPHEREMAP: Begin
         glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
         glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
         glEnable(GL_TEXTURE_GEN_S);
         glEnable(GL_TEXTURE_GEN_T);
      End;

   End; // Of Case
End;
//----------------------------------------------------------------------------
// Mausklicks an ButtonBar weiterleiten
//----------------------------------------------------------------------------
Procedure TMyProgram.HandleMousePressed(event: TSDL_MouseButtonEvent);
Begin
   //Inherited HandleMousePressed(event); // This is empty
   With Event Do MyButtonBar.MouseBtnLDown(X,ScreenHeight-Y);
End;
//----------------------------------------------------------------------------
// Switch per ButtonBar ndern
//----------------------------------------------------------------------------
Procedure TMyProgram.ChangeSwitch(ASender: TObject; AIndex: Integer);
Begin
   Case AIndex Of
      LIGHT_NONE: Switch:= LIGHT_NONE;
      LIGHT_ORDINARY: Switch:= LIGHT_ORDINARY;
      LIGHT_PER_SPOT: Switch:= LIGHT_PER_SPOT;
      LIGHT_PER_SPHEREMAP: Switch:= LIGHT_PER_SPHEREMAP;

      Else Switch:= LIGHT_NONE;
   End;
   TurnLightOFF;
End;
//----------------------------------------------------------------------------
// Zeichnen der Szene
//----------------------------------------------------------------------------
procedure TMyProgram.DrawScene;
Const
   RENDER_SMOOTH = TRUE;
   RENDER_FACES = FALSE;
Var
   Viewport: Array [0..3] Of Integer;
begin
  glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );

  //------------------------------------------------------------------
  // Render model
  //------------------------------------------------------------------
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glGetIntegerv(GL_VIEWPORT, @Viewport);
  gluPerspective(45.0, ViewPort[2]/ViewPort[3], 0.1, 10.0);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;

  If Switch = LIGHT_PER_SPHEREMAP
     Then Textures.Bind
        ('Verlauf');           // Spheremap-Textur einbinden

  With ButtonColor Do glColor4f(Red,Green,Blue,Alpha);

  TurnLightON;                 // STATISCES LICHT EINSCHALTEN

  glTranslatef(0,0,-6);

  glPushMatrix;
     glTranslatef(-1.5,0,0);
     glRotatef(Angle,0,1,0);
     glRotatef(45,1,0,0);

     MyShape3D.Render
        (RENDER_SMOOTH);       // Linker Torus: Normale per Vertex
  glPopMatrix;

  glTranslatef(+1.5,0,0);
  glRotatef(Angle,0,1,0);
  glRotatef(45,1,0,0);

  MyShape3D.Render
     (RENDER_FACES);           // Rechter Torus: Normale per Face

  If Switch = LIGHT_PER_SPHEREMAP
     Then glBindTexture
         (GL_TEXTURE_2D,0);    // Spheremap-Textur "unbinden"

  Inc(Angle);                  // Winkel weiterdrehen
  If Angle=360
     Then Angle:=0;

  //------------------------------------------------------------------
  // Render Buttonbar + FPS
  // Licht und Tiefentest wird fr die Knopfleiste ausgeschaltet
  //------------------------------------------------------------------

   TurnLightOFF;               // STATISCES LICHT AUSSCHALTEN

   glDisable(GL_DEPTH_TEST);   // Tiefentest abschalten

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity;
   glOrtho(0,ViewPort[2],0,ViewPort[3],+1,-1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity;

   MyButtonBar.Render;         // ButtonBar zeichnen

   glEnable(GL_DEPTH_TEST);

  Sleep(20);                   // ein wenig schlafen :)
end;
(******************************************************************************)
(******************************************************************************)
//  Hauptprogramm
(******************************************************************************)
(******************************************************************************)
var
   Color: TSDL_Color;

begin
  // Initalisierung
  MyProgram := TMyProgram.Create;
  MyProgram.WindowCaption := 'www.delphigl.com .: Tutorial 8: LICHT';
  Angle:=0;

  // Zeichen-Parameter setzen
  SetOpenGLParams;

  // Modell erzeugen
  MyShape3D:= TShape3D.Create(stTorus,36,18);
  //MyShape3D:= TShape3D.Create(stSphere,36,18);

  // EasySDL-Font erzeugen
  With Color Do Begin r:=255; g:=255; b:=255; unused:=0; End;
  MyProgram.FPSFont:=TFont.Create
     (DGLSDK_DATA+'fonts/monofont.ttf',18,Color, 0);

  // Knopfleiste erzeugen
  With ButtonColor Do Begin Red:=0; Green:=0; Blue:=1; Alpha:=1; End;
  MyButtonBar:= TButtonBar.Create(ButtonColor,10,470,MyProgram.FPSFont);
  With MyButtonBar Do Begin
     AddButton('No Light');
     AddButton('Ordinary Light');
     AddButton('Spotlight');
     AddButton('Spheremap');

     OnClick:= MyProgram.ChangeSwitch;
     Switch:= LIGHT_NONE;
     SetIndex(Switch);
     TurnLightOFF;
  End;

  // Textur fr Spheremap laden
  Textures.Load('Rotverlauf.bmp','Verlauf');

  // Programmhauptschleife
  MyProgram.Loop;

  // Programmende
  MyButtonBar.Free;
  MyShape3D.Free;
  MyProgram.Free;
end.
